<?xml version="1.0" encoding="ISO-8859-1" standalone="yes"?>
<!--
The Apache Software License, Version 1.1

Copyright (c) 1999, 2000 The Apache Software Foundation.  All rights
reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in
   the documentation and/or other materials provided with the
   distribution.

3. The end-user documentation included with the redistribution, if
   any, must include the following acknowlegement:
      "This product includes software developed by the
       Apache Software Foundation (http://www.apache.org/)."
   Alternately, this acknowlegement may appear in the software itself,
   if and wherever such third-party acknowlegements normally appear.

4. The names "The Jakarta Project", "Ant", and "Apache Software
   Foundation" must not be used to endorse or promote products derived
   from this software without prior written permission. For written
   permission, please contact apache@apache.org.

5. Products derived from this software may not be called "Apache"
   nor may "Apache" appear in their names without prior written
   permission of the Apache Group.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
====================================================================

This software consists of voluntary contributions made by many
individuals on behalf of the Apache Software Foundation.  For more
information on the Apache Software Foundation, please see
<http://www.apache.org/>.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" indent="yes"/>
	<!--
	A boolean flag that tells whether a full stand-alone HTML page should be rendered, or just the body inner content.
	-->
	<xsl:param name="standAloneHTML" select="'true'"/>
	<!--
	Defines the stylesheet URI that will be used by the generated HTML page.
	-->
	<xsl:param name="CSS" select="'stylesheet.css'"/>
	<!--
	Enables to define the first version of the release notes from which the documentation must be generated.
	-->
	<xsl:param name="firstVersion"/>
	<!--
	The last version the documentation must be generated for.
	-->
	<xsl:param name="lastVersion"/>
	<!--
	This enables to filter the items that should be documented (by defaults, all items are documented). Possible values are 'func' or 'tech', i.e. the possible values for the 'func' attribute of the 'Item' XML element in the XML release notes file.
	-->
	<xsl:param name="typeFilter"/>
	<!--
	The minimum value of the XML Item 'visibility' attribute, in order to be rendered.
	-->
	<xsl:param name="visibilityThreshold" select="'4'"/>
	<xsl:template match="/ReleaseNotes">
		<xsl:choose>
			<xsl:when test="$standAloneHTML = 'true'">
				<html>
					<head>
						<title>
							<xsl:text>Release Notes of </xsl:text>
							<xsl:value-of select="@ID"/>
						</title>
						<meta name="Author" content="Edouard Mercier"/>
						<link rel="StyleSheet" type="text/css">
							<xsl:attribute name="href"><xsl:value-of select="$CSS"/></xsl:attribute>
						</link>
					</head>
					<body>
						<xsl:call-template name="_print"/>
					</body>
				</html>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="_print"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="_print">
		<table class="releases" cellspacing="0" cellpadding="10">
			<thead>
				<tr>
					<td colspan="3">
						<xsl:text>The following table shows the history of </xsl:text>
						<b>
							<xsl:value-of select="@ID"/>
						</b>
						<xsl:text> release notes.</xsl:text>
					</td>
				</tr>
			</thead>
			<tbody>
				<tr class="header">
					<th width="15%">Version</th>
					<th>Content</th>
					<th width="15%">Delivery</th>
				</tr>
				<xsl:apply-templates select="Releases"/>
			</tbody>
			<tfoot>
				<tr>
					<td colspan="3">
						<div class="copyright">Release notes generated by <strong>
								<i>
									<a href="http://xmlreleasenotes.free.fr" target="_blank">XMLReleaseNotes</a>
								</i>
							</strong>
						</div>
					</td>
				</tr>
			</tfoot>
		</table>
		<br/>
	</xsl:template>
	<xsl:template match="Releases">
		<xsl:variable name="actual_first_version">
			<xsl:choose>
				<xsl:when test="$firstVersion">
					<xsl:value-of select="$firstVersion"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="Release[last()]/@version"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="actual_last_version">
			<xsl:choose>
				<xsl:when test="$lastVersion">
					<xsl:value-of select="$lastVersion"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="Release[1]/@version"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="first_version_index">
			<xsl:for-each select="Release">
				<xsl:if test="@version = $actual_first_version">
					<xsl:value-of select="position()"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
		<xsl:variable name="last_version_index">
			<xsl:for-each select="Release">
				<xsl:if test="@version = $actual_last_version">
					<xsl:value-of select="position()"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
		<!--xsl:if test="string-length(first_version_index) = 0">
			<xsl:message>Warning, the 'firstVersion' parameter set to '<xsl:value-of select="$firstVersion"/>' seems to be incorrect!</xsl:message>
		</xsl:if>
		<xsl:if test="not(last_version_index)">
			<xsl:message>Warning, the 'lastVersion' parameter set to '<xsl:value-of select="$lastVersion"/>' seems to be incorrect!</xsl:message>
		</xsl:if-->
		<xsl:for-each select="Release">
			<xsl:if test="position() &lt;= $first_version_index and position() &gt;= $last_version_index">
				<tr>
					<xsl:attribute name="class"><xsl:choose><xsl:when test="(position() mod 2) = 0">even</xsl:when><xsl:otherwise>odd</xsl:otherwise></xsl:choose></xsl:attribute>
					<td>
						<b>
							<div class="version">
								<xsl:value-of select="@version"/>
							</div>
						</b>
						<xsl:if test="@buildDate != ''">
							<br/>
							<i>
								<font size="-1">build date: </font>
							</i>
							<span class="date">
								<xsl:value-of select="@buildDate"/>
							</span>
						</xsl:if>
						<xsl:if test="@deliveryDate != ''">
							<br/>
							<i>
								<font size="-1">deliveryDate: </font>
							</i>
							<span class="date">
								<xsl:value-of select="@deliveryDate"/>
							</span>
						</xsl:if>
					</td>
					<td>
						<div class="releaseSummary">
							<xsl:value-of select="Summary"/>
						</div>
						<br/>
						<xsl:apply-templates select="Items"/>
						<xsl:call-template name="_printComposition"/>
						<xsl:call-template name="_printDependencies"/>
						<xsl:call-template name="_printRequirements"/>
					</td>
					<td>
						<xsl:choose>
							<xsl:when test="Deliveries">
								<xsl:apply-templates select="Deliveries"/>
							</xsl:when>
							<xsl:otherwise>No delivery</xsl:otherwise>
						</xsl:choose>
					</td>
				</tr>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	<!--
	Warning, the '_printComposition', '_printRequirements' and '_printDependencies' are the same templates, as far as the logic and layou are concerned. They only differ by the XML element they handle. Should think of a way to collapse them into a single template...
	-->
	<xsl:template name="_printComposition">
		<xsl:variable name="release_current" select="position()"/>
		<xsl:variable name="release_newest">
			<xsl:choose>
				<xsl:when test="Composition">
					<xsl:value-of select="count(preceding-sibling::Release) + 1"/>
				</xsl:when>
				<xsl:when test="count(following-sibling::Release[count(Composition) > 0]) != 0">
					<xsl:value-of select="count((following-sibling::Release[count(Composition) > 0])[1]/preceding-sibling::Release) + 1"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="count(../Release)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="release_oldest">
			<xsl:choose>
				<xsl:when test="count(../Release[position() >= $release_newest and Composition/@inherit = 'false']) != 0">
					<xsl:value-of select="count((../Release[position() >= $release_newest and Composition/@inherit = 'false'])[1]/preceding-sibling::Release) + 1"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="count(../Release)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:if test="$release_newest &lt; count(..) or ../Release[last()]/Composition">
			<br/>
			<table class="composition">
				<thead>
					<tr>
						<th>
							<i>
								<font size="-1">Composition</font>
							</i>
						</th>
					</tr>
				</thead>
				<tbody>
					<xsl:if test="Composition/Summary">
						<tr>
							<th colspan="3">
								<xsl:value-of select="Composition/Summary"/>
							</th>
						</tr>
					</xsl:if>
					<xsl:for-each select="../Release">
						<xsl:variable name="release" select="position()"/>
						<xsl:if test="position() >= $release_newest and position() &lt;= $release_oldest">
							<xsl:for-each select="Composition/Component">
								<xsl:if test="count(../../../Release[position() >= $release_newest and position() &lt; $release]/Composition/Component[@ID = current()/@ID]) = 0">
									<tr>
										<td>
											<xsl:if test="$release = $release_current">[new] </xsl:if>
													[from&#160;<span class="version">
												<xsl:value-of select="../../@version"/>
											</span>]</td>
										<td>
											<xsl:value-of select="@ID"/>
											<xsl:text>&#160;</xsl:text>
											<xsl:value-of select="@version"/>
										</td>
										<td>
											<xsl:if test="text()">
												<xsl:value-of select="text()"/>
											</xsl:if>
										</td>
									</tr>
								</xsl:if>
							</xsl:for-each>
						</xsl:if>
					</xsl:for-each>
				</tbody>
			</table>
		</xsl:if>
	</xsl:template>
	<xsl:template name="_printRequirements">
		<xsl:variable name="release_current" select="position()"/>
		<xsl:variable name="release_newest">
			<xsl:choose>
				<xsl:when test="Requirements">
					<xsl:value-of select="count(preceding-sibling::Release) + 1"/>
				</xsl:when>
				<xsl:when test="count(following-sibling::Release[count(Requirements) > 0]) != 0">
					<xsl:value-of select="count((following-sibling::Release[count(Requirements) > 0])[1]/preceding-sibling::Release) + 1"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="count(../Release)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="release_oldest">
			<xsl:choose>
				<xsl:when test="count(../Release[position() >= $release_newest and Requirements/@inherit = 'false']) != 0">
					<xsl:value-of select="count((../Release[position() >= $release_newest and Requirements/@inherit = 'false'])[1]/preceding-sibling::Release) + 1"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="count(../Release)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:if test="$release_newest &lt; count(..) or ../Release[last()]/Requirements">
			<br/>
			<table class="requirement">
				<thead>
					<tr>
						<th>
							<i>
								<font size="-1">Requirements</font>
							</i>
						</th>
					</tr>
				</thead>
				<tbody>
					<xsl:if test="Requirements/Summary">
						<tr>
							<th colspan="3">
								<xsl:value-of select="Requirements/Summary"/>
							</th>
						</tr>
					</xsl:if>
					<xsl:for-each select="../Release">
						<xsl:variable name="release" select="position()"/>
						<xsl:if test="position() >= $release_newest and position() &lt;= $release_oldest">
							<xsl:for-each select="Requirements/Component">
								<xsl:if test="count(../../../Release[position() >= $release_newest and position() &lt; $release]/Requirements/Component[@ID = current()/@ID]) = 0">
									<tr>
										<td>
											<xsl:if test="$release = $release_current">[new] </xsl:if>
													[from&#160;<span class="version">
												<xsl:value-of select="../../@version"/>
											</span>]</td>
										<td>
											<xsl:value-of select="@ID"/>
											<xsl:text>&#160;</xsl:text>
											<xsl:value-of select="@version"/>
										</td>
										<td>
											<xsl:if test="text()">
												<xsl:value-of select="text()"/>
											</xsl:if>
										</td>
									</tr>
								</xsl:if>
							</xsl:for-each>
						</xsl:if>
					</xsl:for-each>
				</tbody>
			</table>
		</xsl:if>
	</xsl:template>
	<xsl:template name="_printDependencies">
		<xsl:variable name="release_current" select="position()"/>
		<xsl:variable name="release_newest">
			<xsl:choose>
				<xsl:when test="Dependencies">
					<xsl:value-of select="count(preceding-sibling::Release) + 1"/>
				</xsl:when>
				<xsl:when test="count(following-sibling::Release[count(Dependencies) > 0]) != 0">
					<xsl:value-of select="count((following-sibling::Release[count(Dependencies) > 0])[1]/preceding-sibling::Release) + 1"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="count(../Release)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="release_oldest">
			<xsl:choose>
				<xsl:when test="count(../Release[position() >= $release_newest and Requirements/@inherit = 'false']) != 0">
					<xsl:value-of select="count((../Release[position() >= $release_newest and Dependencies/@inherit = 'false'])[1]/preceding-sibling::Release) + 1"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="count(../Release)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:if test="$release_newest &lt; count(..) or ../Release[last()]/Dependencies">
			<br/>
			<table class="dependency">
				<thead>
					<tr>
						<th>
							<i>
								<font size="-1">Dependencies</font>
							</i>
						</th>
					</tr>
				</thead>
				<tbody>
					<xsl:if test="Dependencies/Summary">
						<tr>
							<th colspan="3">
								<xsl:value-of select="Dependencies/Summary"/>
							</th>
						</tr>
					</xsl:if>
					<xsl:for-each select="../Release">
						<xsl:variable name="release" select="position()"/>
						<xsl:if test="position() >= $release_newest and position() &lt;= $release_oldest">
							<xsl:for-each select="Dependencies/Component">
								<xsl:if test="count(../../../Release[position() >= $release_newest and position() &lt; $release]/Dependencies/Component[@ID = current()/@ID]) = 0">
									<tr>
										<td>
											<xsl:if test="$release = $release_current">[new] </xsl:if>
													[from&#160;<span class="version">
												<xsl:value-of select="../../@version"/>
											</span>]</td>
										<td>
											<xsl:value-of select="@ID"/>
											<xsl:text>&#160;</xsl:text>
											<xsl:value-of select="@version"/>
										</td>
										<td>
											<xsl:if test="text()">
												<xsl:value-of select="text()"/>
											</xsl:if>
										</td>
									</tr>
								</xsl:if>
							</xsl:for-each>
						</xsl:if>
					</xsl:for-each>
				</tbody>
			</table>
		</xsl:if>
	</xsl:template>
	<xsl:template match="Deliveries">
		<xsl:value-of select="Summary"/>
		<xsl:choose>
			<xsl:when test="Delivery">
				<ul>
					<xsl:for-each select="Delivery">
						<xsl:variable name="message">
							<xsl:choose>
								<xsl:when test="text()">
									<xsl:value-of select="."/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="@href"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<li>
							<div class="delivery">
								<a>
									<xsl:attribute name="href"><xsl:if test="/ReleaseNotes/Deliveries/@href"><xsl:value-of select="/ReleaseNotes/Deliveries/@href"/><xsl:text>/</xsl:text></xsl:if><xsl:value-of select="@href"/></xsl:attribute>
									<xsl:value-of select="$message"/>
								</a>
							</div>
						</li>
					</xsl:for-each>
				</ul>
			</xsl:when>
			<xsl:otherwise>No delivery</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="Release/Items">
		<xsl:if test="Item">
			<table class="item">
				<tbody>
					<xsl:for-each select="Item[(not(@visibility) or @visibility &gt;= $visibilityThreshold) and (not($typeFilter) or @type = $typeFilter)]">
						<tr>
							<xsl:attribute name="class"><xsl:choose><xsl:when test="(position() mod 2) = 0">even</xsl:when><xsl:otherwise>odd</xsl:otherwise></xsl:choose></xsl:attribute>
							<td>
								<span>
									<xsl:attribute name="class"><xsl:value-of select="@type"/>Type</xsl:attribute>[<xsl:value-of select="@type"/>]</span>
								<xsl:text> </xsl:text>
							</td>
							<td>
								<xsl:call-template name="_printFix"/>
								<xsl:value-of select="."/>
								<xsl:call-template name="_printAskingPersons"/>
								<xsl:call-template name="_printContributors"/>
							</td>
						</tr>
					</xsl:for-each>
				</tbody>
			</table>
		</xsl:if>
	</xsl:template>
	<xsl:template name="_printAskingPersons">
		<xsl:if test="count(@askingPersonID | AskingPersons/AskingPerson/@ID) > 0">
			<div align="right">
				<i>
					<font size="-1">
						<xsl:text>asking: </xsl:text>
					</font>
				</i>
				<span class="askingID">
					<xsl:for-each select="@askingPersonID | AskingPersons/AskingPerson/@ID">
						<xsl:value-of select="."/>
						<xsl:if test="position() != last()">, </xsl:if>
					</xsl:for-each>
				</span>
			</div>
		</xsl:if>
	</xsl:template>
	<xsl:template name="_printContributors">
		<xsl:if test="count(@contributorID | Contributors/Contributor/@ID) > 0">
			<div align="right">
				<i>
					<font size="-1">
						<xsl:text>contributor: </xsl:text>
					</font>
				</i>
				<span class="contributorID">
					<xsl:for-each select="@contributorID | Contributors/Contributor/@ID">
						<xsl:value-of select="."/>
						<xsl:if test="position() != last()">, </xsl:if>
					</xsl:for-each>
				</span>
			</div>
		</xsl:if>
	</xsl:template>
	<xsl:template name="_printFix">
		<xsl:if test="@fixID">
			<xsl:text>fixes bug '</xsl:text>
			<xsl:value-of select="@fixID"/>
			<xsl:text>'</xsl:text>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
